local dkjson = require( "lib.dkjson" )
local love = assert( love )
local shader = require( "shaders.flat" )
local mat = require( "mat4" )

local attributeNames = {
  POSITION = "VertexPosition",
  NORMAL = "VertexNormal",
}

local types = {
  ["SCALAR"] = 1,
  ["VEC3"] = 3,
}

local componentTypes = {
  [5121] = "byte",
  [5123] = "unorm16",
  [5126] = "float",
}

local elementComponentTypes = {
  [5123] = "uint16",
  [5126] = "uint32",
}

local asset
local buffer
do
  local fd = assert( love.filesystem.newFileData( "models/contempra.gltf" ) )
  asset = dkjson.decode( fd:getString() )
  buffer = love.filesystem.newFileData( "models/contempra.bin" )
end

--load all the bufferViews
local bufferViews = {}
for i, view in pairs( asset.bufferViews ) do
  bufferViews[i] = love.data.newByteData( buffer, view.byteOffset, view.byteLength )
end

local function attributeFormat( attributeName, accessor )
  return {attributeNames[attributeName], componentTypes[accessor.componentType], types[accessor.type] }
end

local function lgMesh( primitive )
  assert( not( primitive.mode ) ) --triangles only

  local finalMesh
  for name, accIdx in pairs( primitive.attributes ) do
    local acc = asset.accessors[accIdx+1]
    
    local mesh = love.graphics.newMesh(
      {attributeFormat(name, acc)}, --vertexformat
      acc.count,
      "triangles",
      "static"
    )
    
    local bfv = bufferViews[1+acc.bufferView]
    mesh:setVertices( bfv )
    
    if finalMesh then
      finalMesh:attachAttribute( attributeNames[name], mesh )
    else
      finalMesh = mesh
    end
    
  end
  
  
  do
    local acc = asset.accessors[1+primitive.indices]
    local bfv = bufferViews[1+acc.bufferView]
    finalMesh:setVertexMap(bfv, elementComponentTypes[acc.componentType] )
  end
  return finalMesh
end

local phone = lgMesh( asset.meshes[1].primitives[1] )
local handset = lgMesh( asset.meshes[2].primitives[1] )


local obj = {
  isRinging = false,
  isTalking = false,
  x = 2,
  y = 0.25,
  z = -2,
  size = 0.1,
}

local phoneTF = mat.TRS(
  obj.size,obj.size,obj.size,
  0,0.12,0,
  obj.x, obj.y, obj.z)

local lx, lz = 1, 0
local c, s = math.cos( 0.01 ), math.sin( 0.01 )
function obj.draw( view, proj )

    love.graphics.setColor( 1,0.2,0.2,1 )
    love.graphics.setMeshCullMode( "none" ) --debug
    shader:send( "view", "column", view )
    shader:send( "proj", "column", proj )
    
    lx, lz = c*lx+s*lz,c*lz-s*lx
    shader:send( "light", {0, 1, 0} )
    love.graphics.setShader( shader )
    shader:send( "mdl", "column", phoneTF )
    love.graphics.draw( phone )
    love.graphics.draw( handset )
end

function obj.ring()
  
end

function obj.answer()
  
end

return obj